#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;

namespace HeuristicLab.Optimization.Networks {
  [Item("InputOutputPort", "A synchronous input/output port of an optimization network node.")]
  [StorableClass]
  public class InputOutputPort<TIn, TOut> : Port<TIn>, IInputOutputPort<TIn, TOut>
    where TIn : class, IItem
    where TOut : class, IItem {
    [Storable]
    private IOutputInputPort<TIn, TOut> outputInputPort;
    public IOutputInputPort<TIn, TOut> OutputInputPort {
      get { return outputInputPort; }
      set {
        if (outputInputPort != value) {
          DeregisterOutputInputPortEvents();
          outputInputPort = value;
          RegisterOutputInputPortEvents();
          OnOutputInputPortChanged();
        }
      }
    }
    IOutputInputPort IInputOutputPort.OutputInputPort {
      get { return outputInputPort; }
    }

    [StorableConstructor]
    protected InputOutputPort(bool deserializing) : base(deserializing) { }
    protected InputOutputPort(InputOutputPort<TIn, TOut> original, Cloner cloner) : base(original, cloner) {
      outputInputPort = cloner.Clone(original.outputInputPort);
      RegisterOutputInputPortEvents();
    }
    public InputOutputPort() : base("InputOutputPort") { }
    public InputOutputPort(string name) : base(name) { }
    public InputOutputPort(string name, string description) : base(name, description) { }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterOutputInputPortEvents();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new InputOutputPort<TIn, TOut>(this, cloner);
    }

    public event EventHandler OutputInputPortChanged;
    protected void OnOutputInputPortChanged() {
      var handler = OutputInputPortChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    protected void RegisterOutputInputPortEvents() {
      outputInputPort.ValueSent += OutputInputPort_ValueSent;
    }
    protected void DeregisterOutputInputPortEvents() {
      outputInputPort.ValueSent -= OutputInputPort_ValueSent;
    }
    protected void OutputInputPort_ValueSent(object sender, OutputInputPortEventArgs<TOut> e) {
      Value = OutputInputPort.Value;
      //TODO: switch threads to decouple value propagation and resulting action
      //TODO: clone value?
      //TODO: add computed result to event args
    }
  }
}
