#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Optimization.Networks {
  [Item("Node", "A node of an optimization network.")]
  [StorableClass]
  public class Node : NamedItem, INode {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.RadialChart; }
    }

    [Storable]
    private PortCollection ports;
    protected PortCollection Ports {
      get { return ports; }
    }
    private ReadOnlyKeyedItemCollection<string, IPort> readOnlyPorts;
    IKeyedItemCollection<string, IPort> INode.Ports {
      get {
        if (readOnlyPorts == null) readOnlyPorts = ports.AsReadOnly();
        return readOnlyPorts;
      }
    }

    [StorableConstructor]
    protected Node(bool deserializing) : base(deserializing) { }
    protected Node(Node original, Cloner cloner)
      : base(original, cloner) {
      ports = cloner.Clone(original.ports);
      readOnlyPorts = null;
    }
    public Node()
      : base("Node") {
      ports = new PortCollection();
      readOnlyPorts = null;
    }
    public Node(string name)
      : base(name) {
      ports = new PortCollection();
      readOnlyPorts = null;
    }
    public Node(string name, string description)
      : base(name, description) {
      ports = new PortCollection();
      readOnlyPorts = null;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Node(this, cloner);
    }
  }
}
